const fs = require('fs')
const path = require('path')

const shell = require('shelljs')
const chalk = require('chalk')
const _ = require('lodash')

const Utils = require('../common/utils')

exports.command = 'new <name> [repo] [branch]'
exports.aliases = 'n'
exports.desc = 'Create a new project from specific repo'

exports.builder = function (yargs) {
  yargs.option('yarn', {
    default: false,
    describe: 'use yarn command'
  })

  yargs.option('yes', {
    alias: 'y',
    default: false,
    describe: 'run npm/yarn init with --yes'
  })

  yargs.option('force', {
    alias: 'f',
    default: false,
    describe: 'force download, existed folder will be deleted!'
  })

  yargs.option('merge', {
    alias: 'm',
    default: false,
    describe: 'merge config with exist project folder!'
  })

  yargs.option('empty', {
    default: false,
    describe: 'force empty project, ignore repo'
  })

  yargs.option('add', {
    default: false,
    describe: 'add npm package to package.json dependencies'
  })

  yargs.option('add-dev', {
    default: false,
    describe: 'add npm package to package.json devDependencies'
  })

  yargs.option('init', {
    default: false,
    describe: 'init the project use Zignis'
  })
}

exports.handler = function (argv) {
  argv.repo = argv.repo || _.get(Utils.getCombinedConfig(), 'commandDefault.new.repo') || ''
  argv.branch = argv.branch || _.get(Utils.getCombinedConfig(), 'commandDefault.new.branch') || 'master'

  if (fs.existsSync(path.resolve(process.cwd(), argv.name))) {
    if (argv.force) {
      Utils.warn(`Existed ${argv.name} is deleted before creating a new one!`)
      shell.rm('-rf', path.resolve(process.cwd(), argv.name))
    } else if (!argv.merge) {
      Utils.error(`Destination existed, command abort!`)
    }
  }

  if (argv.merge) {
    shell.cd(argv.name)
  } else {
    if (!argv.repo || argv.empty) {
      shell.mkdir('-p', path.resolve(process.cwd(), argv.name))
      shell.cd(argv.name)
      if (argv.yarn) {
        if (argv.yes) {
          shell.exec('yarn init -y')
        } else {
          shell.exec('yarn init')
        }
      } else {
        if (argv.yes) {
          shell.exec('npm init -y')
        } else {
          shell.exec('npm init')
        }
      }
    } else {
      console.log(chalk.green(`Downloading from ${argv.repo}`))
      try {
        shell.exec(`git clone ${argv.repo} ${argv.name} --single-branch --depth=1 --branch ${argv.branch} --progress`)

        console.log(chalk.green('Succeeded!'))
        shell.cd(argv.name)
        shell.rm('-rf', path.resolve(process.cwd(), `.git`))
        console.log(chalk.green('.git directory removed!'))
        if (argv.yarn) {
          shell.exec('yarn')
        } else {
          shell.exec('npm install')
        }

        shell.exec('git init')
        console.log(chalk.green('New .git directory created!'))
      } catch (e) {
        Utils.error(e.message)
      }
    }
  }

  // add packages
  const addPackage = Utils.parsePackageNames(argv.add)
  const addPackageDev = Utils.parsePackageNames(argv.addDev)
  if (addPackage.length > 0) {
    if (argv.yarn) {
      shell.exec(`yarn add ${addPackage.join(' ')}`)
    } else {
      shell.exec(`npm install ${addPackage.join(' ')}`)
    }
  }

  if (addPackageDev.length > 0) {
    if (argv.yarn) {
      shell.exec(`yarn add ${addPackageDev.join(' ')} -D`)
    } else {
      shell.exec(`npm install ${addPackageDev.join(' ')} --save-dev`)
    }
  }

  // init basic zignis structure
  if (argv.init) {
    if (argv.name.indexOf('zignis-plugin-') === 0) {
      shell.exec('zignis init --plugin --disable-ten-temporarily --force')
    } else {
      shell.exec('zignis init --disable-ten-temporarily --force')
    }
  }
}
